/* 	DefaultAction.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 20-feb-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Jun 11, 2007	dennis@brains2b.nl      Added constructur for action name only		
*/
package org.brains2b.swingext.action;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.KeyStroke;
import javax.swing.event.SwingPropertyChangeSupport;

/** Default action implements Action and gives access to all
 *  Action properties.
 *  <p>This implementation is based on a flattend Object array.</p> 
 *  <p>All actions are send directly to the ActionListener in this action.
 *  You do not have to set the ActionListener directly. Adding this action to
 *  {@link ExtActionMap} will fill the listener automatically from ActionMap</p>
 *  
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [11-jun-2007]
 */
public class DefaultAction implements Action {
    
    private Object[] m_array =new Object[0];
    private ActionListener m_listener;
    private SwingPropertyChangeSupport m_changeSupport;
    private boolean m_enabled=true;
    
    public DefaultAction(ActionListener listener) {
        super();
        m_listener = listener;
    }
    
    public DefaultAction(ActionListener listener, Object action) {
        this(listener);
        putValue(ACTION_COMMAND_KEY, action);
    }
    
    public DefaultAction(ActionListener listener, Object action, String label, String description,Icon icon, KeyStroke stroke, char mnemonic ) {
        this(listener);
        putValue(ACTION_COMMAND_KEY, action);
        putValue(NAME, label);
        putValue(SHORT_DESCRIPTION,description);
        putValue(SMALL_ICON, icon);
        putValue(ACCELERATOR_KEY,stroke);
        putValue(MNEMONIC_KEY,new Integer(mnemonic));
    }
    
    /**
     * {@inheritDoc}
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        if (m_changeSupport==null) {
            m_changeSupport=new SwingPropertyChangeSupport(this);
        }
       m_changeSupport.addPropertyChangeListener(listener);
    }

    /**
     * {@inheritDoc}
     */
    public Object getValue(String key) {
        for (int i=0;i<m_array.length;i+=2) {
            if (key.equals(m_array[i])) {
                return m_array[i+1];            
            }
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isEnabled() {
        return m_enabled;
    }

    /**
     * {@inheritDoc}
     */
    public void putValue(String key, Object value) {
        if (key==null) return;
        Object oldValue=getValue(key);
        if (oldValue==null && value==null) return;
        if (oldValue==null) {
            int length=m_array.length;
            resize(length,2);
            m_array[length]=key;
            m_array[length+1]=value;
        } else {
            int idx = indexOf(key);
            if (value==null) {
                resize(idx,-2);
            } else {
                m_array[idx+1]=value;
            }
        }
        
        firePropertyChanged(key,oldValue,value);

    }
    
    /**
     * {@inheritDoc}
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        if (m_changeSupport==null) {
           return; 
        }
        m_changeSupport.removePropertyChangeListener(listener);
        
    }

    /**
     * {@inheritDoc}
     */
    public void setEnabled(boolean b) {
        boolean oldValue=m_enabled;
        if (b!=oldValue) {
            m_enabled=b;        
            firePropertyChanged("enabled", Boolean.valueOf(oldValue),Boolean.valueOf(b));
        }
    }

    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
       m_listener.actionPerformed(e);

    }
    
    private void firePropertyChanged(String key, Object oldValue, Object newValue) {
        if (m_changeSupport==null ||
           (oldValue!=null && newValue!=null && oldValue.equals(newValue))) {
            return;
        }
        m_changeSupport.firePropertyChange(key, oldValue, newValue);        
    }
    
    private final int indexOf(final String key) {
        for (int i=0;i<m_array.length;i+=2) {
            if (key.equals(m_array[i])) {
                return i;      
            }
        }
        return -1;
    }
     
    private final void resize(int pos, int change) {
         int length=m_array.length;
         if (change<0) {
             m_array[pos] = m_array[length+change];
             m_array[pos+1] = m_array[length+change+1];
         }
         Object[] tmp =new Object[length+change];
         System.arraycopy(m_array, 0, tmp, 0, change<0?tmp.length:length);
         m_array=tmp;
     }

}
