/* 	ExtActionMap.java
 *	Copyright (c) 2002-2006, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 24-06-2002	dennis@brains2b.org Changed system.close to system.exit
 * 04-12-2002	dennis@brains2b.org Changed map implementation to keep original
 * 														add order.
 * 15-01-2002	dennis@brains2b.org	Changed actionCommand to be implemented as Object
 * 20-09-2003	dennis@brains2b.nl	implement setUnrestriced through this Map
 * 28-11-2006   dennis@brains2b.nl  implemented method to access underlying Map
 */
package org.brains2b.swingext.action;

import java.awt.event.ActionListener;
import java.util.Set;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import org.brains2b.collection.SortedMap;
import org.brains2b.swingext.gui.IconFactory;

/** An Abstract ActionMap with an easy way to add actions
 *  <p>This actionMap takes an ActionListener that handles 
 *  all actions in this Map.</p> 
 *  <p>This actionmap is based on SortedMap to allow for keys returned 
 *  in the order where they where added in.</p>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.32 [06-07-2007]
 * @see org.brains2b.collection.SortedMap
 * @see DefaultAction
 * @see javax.swing.ActionMap
 */
public class ExtActionMap extends ActionMap {

	private ActionListener m_parent;
	private SortedMap m_map = new SortedMap();

	/**
	 * Create an FwActionMap.
	 * @param parent ActionHandler, the ActionHandler handling fired actions.
	 */
	public ExtActionMap(ActionListener parent) {
		super();
        if (parent==null) {
            throw new IllegalArgumentException("Cannot add null as ActionListener");
        }
		m_parent = parent;
	}


	/**
	 * sets Default system actions.
	 * <UL>The following actions are added default
	 * <LI> Exit, with alt-F4 key, actionCommand: <strong> system.close </strong>
	 * <LI> Preferences, no default assignment, actionCommand: <strong> system.preference </strong>
	 * <LI> Help, F1 key, actionCommand <strong> system.help </strong>
	 * <LI> About, No default assignment, actionCommand <strong> system.about </strong>
	 * </UL>
	 */
	public final static void setDefaultSystemActions(ExtActionMap map) {
		//System actions
		map.addAction("System.exit", "Exit", "Exit", null,"alt F4", 'x');
        map.addAction("System.preference", "Preferences", "Show preference dialog",null,null,'p');
        //Help actions
		map.addAction("Help.help", "Help", "Help", null,"F1", 'h');
		map.addAction("Help.about","About", "About", null,null, 'a');

	}

	/**
	 * adds an Action to the Map
     * @param actionCommand String, the actionCommand to perform, also the key with which to find the action in the map
	 * @param label String, the name that will show up as label
	 * @param description String, the description displayed as tooltip.
	 * @param icon javax.swing.Icon, an optional image to display with this action
	 * @param accelerator String , accelarator key as String, refer to <a href="javax.swing.KeyStroke">KeyStroke</a> for a description of KeyStrokes as text.
     * @param mnemomic char, the mnemonic or shortcut key used.
     
	 */
	public void addAction(
		Object actionCommand,
        String label,
		String description,
		String icon,
        String accelerator,
        char mnemomic
        ) {

        if (actionCommand==null) {
            throw new IllegalArgumentException(Action.ACTION_COMMAND_KEY +" cannot be null");
        }
		KeyStroke key = null;
		if (accelerator != null) {
			key = KeyStroke.getKeyStroke(accelerator);
		}
		ImageIcon icn = null;
		if (icon != null) {
            String str = icon;     
            if (icon.indexOf('/')==-1) {
			 str= "/org/brains2b/images/" + icon;
            }
			try {
				icn = IconFactory.createIcon(str);
			} catch (NullPointerException npex) {
				icn = null;
			}
		}

		DefaultAction act =
			new DefaultAction(
				m_parent,
                actionCommand,
				label,
                description,
				icn,
                key,
                mnemomic);
		m_map.put(actionCommand, act);

	}
	
	/**
	 * get the ActionListener belonging to this map
	 * @return ActionListener, returns the handler belonging to this map
	 */
	public ActionListener getListener() {
		return m_parent;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object[] allKeys() {
		Object[] keys = keys();
        if (getParent()==null) {
            return keys;
        }
        Object[] parentKeys = getParent().allKeys();
        if (parentKeys==null || parentKeys.length==0) {
            return keys;
        }
        
        Object[] outKeys= new Object[keys.length+parentKeys.length];
        System.arraycopy(parentKeys, 0, outKeys, keys.length, parentKeys.length);
        System.arraycopy(keys, 0, outKeys, 0, keys.length);
        return outKeys;
	}

	/**
	 * {@inheritDoc}
	 */
	public void clear() {
		m_map.clear();
	}

	/**
	 * {@inheritDoc}
	 */
	public Action get(Object key) {
		Action a = (Action) m_map.get(key);
        if ( a==null && getParent()!=null ) {
            a = getParent().get(key);
        }
        return a;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object[] keys() {
		Set s = m_map.keySet();
        if (s.isEmpty()) return null;
        return s.toArray();
	}

	/**
	 * {@inheritDoc}
	 */
	public void put(Object key, Action action) {
		m_map.put(key, action);
	}

	/**
	 * {@inheritDoc}
	 */
	public void remove(Object key) {
		m_map.remove(key);
	}

	/**
	 * {@inheritDoc}
	 */
	public int size() {
		return m_map.size();
	}

    /**
     * get the SortedMap this ActionMap is based on 
     * @return SortedMap
     */
    public SortedMap getMap() {
        return m_map;
    }

}