/* 	UserProperties.java
 * 
 * Created on: 01/02/2002
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002,2003 Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* 
 * Changelog
 * 02/22/2002 	DeGr 				Implemented default file path {user.home}/.java/conf
 * 									for properties
 * 10/04/2002	dennis@brains2b.nl	overwritten setProperty to check for Dirty
 * 11/17/2002		"		"		Added (file) constructor to not write to defaultpath
 * 13-02-2003		"		"		Used Logger instead of System.out en System.err, 
 * 									standardized member names
 * 16-05-2004		"		"		fixed checkDefaultPath for Windows
 * 23-09-2004		"		"		Only try loading a property file if it exists
 * 10-09-2006		"		"		Save properties in user.dir if user.home is not available
 * TODO get[type]Property functions
 */
package org.brains2b.swingext.conf;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import org.brains2b.log.Logger;

/** Abstract class extending Properties to use for user preferences
 *  in an application. 
 *  <UL>Features include:
 *  <LI>creating a default dir for properties ("user.home"/.java/conf)
 *  <LI>possibility to implement default values for properties
 *  <LI>creating a file for properties if it does not exist.
 *  <LI>writing properties to file if and only if they have changed.
 *	</UL>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.21.2 [23-09-2004]
 * @see java.util.Properties
 */
public class UserPreference extends Properties {
	
	/** Constant defining the default path "user.home"/.java/conf as a String */
	public static final String DEFAULTPATH=System.getProperty("user.home")+ "/.java/conf";
	
	private boolean m_dirty;
	private File m_file;
	
	
    /**
     * Constructor for UserProperties.
     * @param filename String, the filename or relative filepath,including filename
     * to DEFAULTPATH. 
     * <p>The file does not have to exist. Be aware that if you define
     * a relative path this path will be created if properties need to be saved
     * and path does not exist already. Be aware that you need OS access rights 
     * to the location you specified for the properties to be saved.
     * <p>If you want to define the complete path yourself please use UserProperties(File)
     *  
     */
	public UserPreference(String filename) {
		this(new File(DEFAULTPATH,filename));
	}
	
    /**
     * Constructor for UserProperties.
     * @param file File, file object to a properties location. 
     * <p>The file does not have to exist. Be aware that if you define
     * a path this path will be created if properties need to be saved
     * and path does not exist already. Be aware that you need OS access rights 
     * to the location you specified for the properties to be saved.
     * 
     */
	public UserPreference(File file) {
		super();
		m_file=file;
        load();
	}
	
	
    /**
     * loads the properties from the defined file.
     */
	public void load() {
		try {
			if (m_file.exists()) {
			    FileInputStream fis=new FileInputStream(m_file);
			    load(fis);
			    setDirty(false);
			}
		} catch (IOException ioex) {
			Logger.printException(ioex);
			setDirty(true);	
		}
	}
	
	public void store() {
		if(isDirty()) {
			try {
				checkDefaultPath();
				FileOutputStream fos=new FileOutputStream(m_file);
				store(fos,m_file.getCanonicalPath());
			} catch (IOException ioex) {
				Logger.printException(ioex);
			}	
		}
	}
	
	/**
     * {@inheritDoc} 
	 */
	public synchronized Object setProperty(String key, String value) {
		if (key==null || value==null) return null;
		String oldValue=super.getProperty(key);
		if (oldValue==null || !oldValue.equals(value)) {
			setDirty(true);
		}
		return super.put(key, value);
		
	}
    
    /** sets a property as default
     * <p>This means that if a value already exists it is not
     * overwritten and isDirty will stay false</p>
     * 
     * @param key String, the key for this property
     * @param defaultValue String, the defaultValue to set
     */
    public void setDefault(String key, String defaultValue) {
        if (key==null || defaultValue==null) return;
        String oldValue=super.getProperty(key);
        if (oldValue==null) {
            super.put(key,defaultValue);
        }
        
    }


    /**
     * add or modify a property to these UserProperties.
     * <p>prefered is using setProperty instead of this function.
     * Overwritten to track changes.
     */
	public synchronized Object put(Object key,Object value) {
		setDirty(true);
		return super.put(key,value);
	}

		
	/**
	 * Checks to see if the properties have changed.
	 * @return Returns a boolean
	 */
	protected boolean isDirty() {
		return m_dirty;
	}

	/**
	 * Mark these properties as having been changed.
	 * @param dirty The dirty to set
	 */
	protected void setDirty(boolean dirty) {
		m_dirty = dirty;
	}

	protected void setFile(File f) {
		m_file=f;
	}

	/**
	 * gets the filename with its path that is used to store the properties
	 * @return String representation of the complete filepath for the properties
	 */
	public String getFilename() {
		return m_file.getAbsolutePath();
	}

	private void checkDefaultPath() {
		if (DEFAULTPATH.equals(m_file.getParent()) && !m_file.getParentFile().exists()) {
			if (!m_file.getParentFile().mkdirs()) {
				m_file=new File(m_file.getName());
			}
		}
    }			
	
}

