/* 	EasyForm.java
 *	Copyright (c) 2002, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]			[Who]							[What]
 * 03-01-2003	dennis@brains2b.org	Removed get/setObject moved to Control
 * 27-08-2007	dennis@brains2b.org	Allow to set the buttons to show
 * TODO EasyForm needs to be expanded
 */
package org.brains2b.swingext.form;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 * EasyForm is a class for making standardized forms.
 * <UL>Functionality includes
 *  <LI>Support for modes (ADD|EDIT|DELETE|VIEW|SEARCH)</LI>
 *  <LI>Easy way to add components in a GridBagLayout</LI>
 *  <LI>Recalculation of Dimension on the basis of {@link JTextField#setColumns()}</LI>
 *  <LI>Easy retrieval of components by name</LI>
 *  <LI>Correction on setEnabled that will not gray-out text, only background
 * </UL>
 * @version 1.0 [03-01-2003]
 * @author  <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 */
public class EasyForm extends JPanel implements Form{

	public static final int ADD = 0;
	public static final int UPDATE = 1;
	public static final int DELETE = 2;
	public static final int VIEW = 3;
	public static final int SEARCH=5;

	private String m_title;
	private int m_mode;
	private JPanel m_main;
	private FormAction m_action;
	private boolean m_cancel=false;

	public EasyForm(int mode) {
		this(mode, FormAction.BTN_ALL);
	}
	public EasyForm(int mode, int actionButtons) {
		super(new BorderLayout());
        UIManager.put("TextField.inactiveForeground",UIManager.getColor("TextField.foreground"));
		setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
		m_mode = mode;
		defaultInit(actionButtons);
	}

	private void defaultInit(int actionButtons) {
		m_main = new JPanel(new GridBagLayout());
		add(m_main);

		m_action =
			new FormAction(
				this,
				actionButtons);
		add(m_action, BorderLayout.SOUTH);

	}

	public void addField(String label, JComponent cmp, int x, int y, int w) {
	    JLabel l=new JLabel(label);
	    l.setLabelFor(cmp);
		addComponent(l, x, y,1,1);
		addComponent(cmp, x + 1, y, Math.max(w - 1, 1), 1);
	}
	
	public void addComponent(JComponent cmp, int x, int y) {
		addComponent(cmp, x, y, 1, 1);
	}

	public void addComponent(JComponent cmp, int x, int y, int w, int h) {
		m_main.add(cmp, getConstraints(x, y, w, h));
	}

	private GridBagConstraints getConstraints(int x, int y, int w, int h) {
		return new GridBagConstraints(
			x,
			y,
			w,
			h,
			0d,
			0d,
			GridBagConstraints.WEST,
			GridBagConstraints.NONE,
			new Insets(3, 3, 3, 3),
			0,
			0);
	}

	public boolean isCancelled() {
		return m_cancel;
	}
	
	/*
	 * @see Form#performCancel()
	 */
	public void performCancel() {
		m_cancel=true;
		close();
	}

	protected void close() {
		Component parent = getRootPane().getParent();
		if (parent instanceof JFrame) {
			((JFrame) parent).dispose();
		} else if (parent instanceof JDialog) {
			((JDialog) parent).dispose();
		} else if (parent instanceof JInternalFrame) {
			((JInternalFrame) parent).dispose();
		} else {
			parent.setVisible(false);
		}
	}

	protected void handleException(Exception ex) {
		JOptionPane.showMessageDialog(
			this,
			ex.getMessage(),
			"Error",
			JOptionPane.ERROR_MESSAGE);
	}

	/*
	 * @see Form#performOk()
	 */
	public void performOk() {
		performApply();
		close();
	}

	/**
	 * Gets the title.
	 * @return Returns a String
	 */
	public String getTitle() {
		return m_title;
	}

	/**
	 * Sets the title.
	 * @param title The title to set
	 */
	protected void setTitle(int mode, String title) {
		String str=null;
		
		switch (mode) {
			case ADD:
				str="<New> " + title;
				break;
			default: 
				str="" + title;
		}
		
		m_title = str;
	}

	public void setEnabled(JComponent cmp,boolean b) {
        cmp.setEnabled(b);
		cmp.setOpaque(b);
	}
	
	public JPanel getMain() {
	    return m_main;
	}

    public int getMode() {
        return m_mode;
    }

    public void setMode(int mode) {
        m_mode = mode;
    }

    public void performApply() {
    }

    public void performUndo() {
    }
}