/* 	FormAction.java
 *	Copyright (c) 1999-2001, Dennis Groenendijk
 *  Copyright (c) 2002,2003, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* @#History
 * [When]		[Who]				[What]
 * 23-01-2002	DeGr				Added button option BTN_ALL
 * 05-05-2002 	DeGr				Added support for ResourceBundle
 * 20-12-2002	dennis@brains2b.nl  overwritten setFont to apply to all buttons.
 * 12-02-2003	dennis@brains2b.nl overwritten setEnabled to apply to all the buttons when called on parent.
 * 													expanded javadoc comments
 * 27-04-2003	dennis@brains2b.nl	Undo should not validate on lost focus
 * 02-05-2003	dennis@brains2b.nl	Added setDefaultButton, getButton function
 * 31-03-2003	dennis@brains2b.nl	Form(parent) did not call right buttons
 */
package org.brains2b.swingext.form;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

/** Sub panel for default [Ok][Cancel][Undo][Apply] buttons
 *  <p> Can be used in any panel that implements  org.brains2b.model.gui.Form
 *  <p> To set the buttons you want to display add the constants of the buttons you want displayed when invoking the Constructor
 *  <BR>for example:
 * <code>FormAction.BTN_OK_CANCEL+FormAction.UNDO</code>
 * <BR>Shows the [Ok][Undo] and [Cancel] button but not the [Apply] button
 * <BR>	FormAction.BTN_ALL is just a conveniance for displaying all buttons.
 * 
 * <p> You can call the Constructor(Form, buttons, ResourceBundle) to make the labels of the buttons
 * application or language specific. Just pass the appropriate resource bundle to the Constructor
 * given that they define the following keys and values
 * <UL> expected keys:
 * <LI> button Ok: 		FormAction.btnOkText and FormAction.btnOkTip
 * <LI> button Cancel: 	FormAction.btnCancelText and FormAction.btnCancelTip
 * <LI> button Undo: 	FormAction.btnUndoText and FormAction.btnUndoTip
 * <LI> button Apply: 	FormAction.btnUndoText and FormAction.btnApplyTip
 * </UL>
 * <p><i>Note:</i>If one of the keys is not defined there will be no ResourceException, but the default English
 * text will be used.
 * @author <A MAILTO="dennis@brains2b.nl">dennis</A>
 * @version 0.32 [02-05-2003]
 * @see org.brains2b.swingext.gui.Form
 */
public class FormAction extends JPanel {
	
	public final static int BTN_OK=1;
	public final static int BTN_CANCEL=2;
	
	/** Constant for displaying the Ok and Cancel buttons*/
	public final static int BTN_OK_CANCEL=3;
	/** Constant for displaying the Applyl buttons*/
	public final static int BTN_APPLY=4;
	/** Constant for displaying the Undo buttons*/
	public final static int BTN_UNDO=8;
	/** Constant for displaying all the buttons (Ok,Apply,Undo and Cancel)*/
	public final static int BTN_ALL=15;
	
	private class ActionEventHandler implements ActionListener {
		/*
		 * @see ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource()==m_ok) {
				m_form.performOk();
			} else if (e.getSource()==m_undo) {
				m_form.performUndo();
			} else if (e.getSource()==m_apply) {
				m_form.performApply();
			} else if (e.getSource()==m_cancel) {
				m_form.performCancel();
			}
 
		}

	}
	
	
	private JButton m_ok;
	private JButton m_undo;
	private JButton m_apply;
	private JButton m_cancel;
	
	private Form m_form;
	
	/**
	 * Constructor for FormAction.
	 * <p> Displays only the [Ok] and [Cancel] button
	 * <p> The default English labels are used for the buttons
	 * @param form Form, the interface that will define what to do if a button is pressed
	 */
	public FormAction(Form form) {
		this(form,BTN_OK_CANCEL);
	}
	
	/**
	 * Constructor for FormAction.
	 * <p> the default English labels are used for the buttons
	 * @param parent Form, the interface that will define what to do if a button is pressed
	 * @param buttons int, a combination of buttons (see the Constants for this class) to display
	 */
	public FormAction(Form form, int buttons) {
		super();
		m_form=form;
		init();
		showButtons(buttons);
	}
	
	/**
	 * Constructor for FormAction.
	 * @param parent Form, the interface that will define what to do if a button is pressed
	 * @param buttons int, a combination of buttons (see the Constants for this class) to display
	 * @param msg ResourceBundle, the resource bundle to define the labels for the buttons
	 */
	public FormAction(Form form, int buttons, ResourceBundle msg) {
		this(form,buttons);
		setLSText(msg);
	}

	/**
	 * overwritten setFont to apply to all buttons on the Panel as well
	 * as to the Panel itself.
     * @see javax.swing.JComponent#setFont(Font)
     */
    public void setFont(Font font) {
        super.setFont(font);
        Component[] cmps=getComponents();
        for (int i=0;i<cmps.length;i++) {
        	cmps[i].setFont(font);
        }
    }
    
    /**
     * overwritten default setEnabled to propagate setEnabled to all buttons 
     * when setEnabled is called on the panel
     * @param enabled boolean
	 * @see javax.swing.JComponent#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		m_apply.setEnabled(enabled);
		m_ok.setEnabled(enabled);
		m_cancel.setEnabled(enabled);
		m_undo.setEnabled(enabled);
	}
	
	public void setEnabled(int buttonId, boolean enabled) {
		
		if ((buttonId&BTN_UNDO)==BTN_UNDO) 	m_undo.setEnabled(enabled);
		if ((buttonId&BTN_CANCEL)==BTN_CANCEL)	m_cancel.setEnabled(enabled);
		if ((buttonId&BTN_OK)==BTN_OK) 		m_ok.setEnabled(enabled);
		if ((buttonId&BTN_APPLY)==BTN_APPLY) 	m_apply.setEnabled(enabled);
	}

    /**
     * returns the JButton Object on the basis of the id
     * @param buttonId int
     * @return JButton
     * @since 0.32 [2-5-2003]
     */
	 public JButton getButton(int buttonId) {
		if ((buttonId&BTN_UNDO)==BTN_UNDO) 	return m_undo;
		if ((buttonId&BTN_CANCEL)==BTN_CANCEL)	return m_cancel;
		if ((buttonId&BTN_OK)==BTN_OK) 		return m_ok;
		if ((buttonId&BTN_APPLY)==BTN_APPLY) 	return m_apply;
		return null;
	}
	
    /**
     * Sets one of the defined buttons as default.
     * <p>Only use this function after registering it to 
     * a container. Otherwise no action will be taken.
     * 
     * @param buttonId int, One of the buttons as defined by the constants.
     * @since 0.32 [2-5-2003]
     */
	public void setDefaultButton(int buttonId) {
		if (m_form==null || m_form.getRootPane()==null) return;
		m_form.getRootPane().setDefaultButton(getButton(buttonId));
	}
	
	private void init() {
		ActionEventHandler aeh=new ActionEventHandler();
		setBorder(new EmptyBorder(10,5,5,5));
		setLayout(new BoxLayout(this,BoxLayout.X_AXIS));

		// Implement Ok button
		m_ok=new JButton("Ok");
		m_ok.setMnemonic('o');
		m_ok.addActionListener(aeh);
		add(m_ok);
		// Implement Apply button
		m_apply=new JButton("Apply");
		m_apply.setMnemonic('a');
		m_apply.addActionListener(aeh);
		add(m_apply);
		//Implement dummy panel
		add(new JPanel());
		
		//Implement Undo button
		m_undo=new JButton("Undo");
		m_undo.setMnemonic('u');
		m_undo.setVerifyInputWhenFocusTarget(false);
		m_undo.addActionListener(aeh);
		add(m_undo);
		//Implement Cancel button
		m_cancel=new JButton("Cancel");
		m_cancel.setMnemonic('c');
		m_cancel.setVerifyInputWhenFocusTarget(false);
		m_cancel.addActionListener(aeh);
		add(m_cancel);
		
	}	
	
	private void showButtons(int buttons) {
		
		m_undo.setVisible((buttons&BTN_UNDO)==BTN_UNDO?true:false);
		m_apply.setVisible((buttons&BTN_APPLY)==BTN_APPLY?true:false);
		m_ok.setVisible((buttons&BTN_OK)==BTN_OK?true:false);
		m_cancel.setVisible((buttons&BTN_CANCEL)==BTN_CANCEL?true:false);
	}
	
	/**
	 * set Language/Application specific text
	 * The used resource bundle is supplied by the application
	 * using this Class.
	 * <p> Missing resource exceptions are ignored and an individual button will
	 * use default english, with no tooltips, if resource is not found
	 * <UL> expected keys:
	 * <LI> button Ok: 		FormAction.btnOkText and FormAction.btnOkTip
	 * <LI> button Cancel: 	FormAction.btnCancelText and FormAction.btnCancelTip
	 * <LI> button Undo: 	FormAction.btnUndoText and FormAction.btnUndoTip
	 * <LI> button Apply: 	FormAction.btnUndoText and FormAction.btnApplyTip
	 * </UL>
	 * @param msg ResourceBundle supplied through Constructor.
	 */
	private void setLSText(ResourceBundle msg) {
		try {
			m_ok.setText(msg.getString("FormAction.btnOkText"));
			m_ok.setToolTipText(msg.getString("FormAction.btnOkTip"));
			m_ok.setMnemonic(msg.getString("FormAction.btnOkMnem").charAt(0));
		} catch (MissingResourceException ex) {}
		try {
			m_undo.setText(msg.getString("FormAction.btnUndoText"));
			m_undo.setToolTipText(msg.getString("FormAction.btnUndoTip"));
			m_undo.setMnemonic(msg.getString("FormAction.btnUndoMnem").charAt(0));
		} catch (MissingResourceException ex) {}
		try {
			m_apply.setText(msg.getString("FormAction.btnApplyText"));
			m_apply.setToolTipText(msg.getString("FormAction.btnApplyTip"));
			m_apply.setMnemonic(msg.getString("FormAction.btnApplyMnem").charAt(0));
		} catch (MissingResourceException ex) {}
		try {
			m_cancel.setText(msg.getString("FormAction.btnCancelText"));
			m_cancel.setToolTipText(msg.getString("FormAction.btnCancelTip"));
			m_cancel.setMnemonic(msg.getString("FormAction.btnCancelMnem").charAt(0));
		} catch (MissingResourceException ex) {}
	}
}

