/* 	ExtButtonGroup.java
 *	Copyright (c) 2002-2007 Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 05-07-2007   dennis@brains2b.nl  Rewritten most of this class to deal 
 *                                  with changed focus on jdk 1.4 and above
 *                                  and no longer require (broken) code for next focus Item 
 */
package org.brains2b.swingext.gui;

import java.awt.KeyboardFocusManager;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Iterator;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;

/** Extended ButtonGroup overwrites ButtonGroup to keep access to
 *  the buttons contained and the sequence it is contained in.
 *  <p>Buttons added to this group will have their focus keys overwritten
 *  for Tab/Shift-Tab to go to the next/previous item in the group.</p>
 *  <p>Left/Down goes to the previous item, Right/Up to the next item within
 *  the group.
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.20 [05-07-2007]
 */
public class ExtButtonGroup extends ButtonGroup {

    class KeyEventHandler extends KeyAdapter {
        public void keyPressed(KeyEvent e) {
            if (e.getKeyCode()==KeyEvent.VK_TAB) {
                KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
                if (e.isShiftDown()) {
                    kfm.focusPreviousComponent((AbstractButton) buttons.firstElement());
                } else {
                    kfm.focusNextComponent((AbstractButton) buttons.lastElement());
                }
            } else if (e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_DOWN) {
                AbstractButton btn = (AbstractButton) e.getSource();
                AbstractButton prev = previous(btn);
                if (prev!=null) prev.requestFocusInWindow();
                
            } else if (e.getKeyCode() == KeyEvent.VK_RIGHT || e.getKeyCode()==KeyEvent.VK_UP ) {
                AbstractButton btn = (AbstractButton) e.getSource();
                AbstractButton next = next(btn);
                if (next!=null) next.requestFocusInWindow();
                
            }
        }
    }
	
	private String m_toolTipText;
    private KeyEventHandler m_keyHandler;
	/**
	 * Constructor for ExtButtonGroup.
	 */
	public ExtButtonGroup() {
		super();
        m_keyHandler = new KeyEventHandler(); 
	}
	
	/**
	 * Add will add an AbstractButton to the ButtonGroup,
	 * set the tooltip text for this group if the button does
     * not have a separate one
	 * <p>Adds a {@link KeyListener} to this button to handle
     * focus keys.  
	 * @see javax.swing.ButtonGroup#add(AbstractButton)
	 */
	public void add(AbstractButton b) {
		super.add(b);
		
		if (b.getToolTipText()==null) {
			b.setToolTipText(m_toolTipText);
		}
		
		b.setFocusTraversalKeysEnabled(false);
        b.addKeyListener(m_keyHandler);
	}

	/**
	 * Removes the button from the ButtonGroup
	 * <p>Removes the {@link KeyListener} from the button</p>
	 * @see javax.swing.ButtonGroup#remove(AbstractButton)
	 */
	public void remove(AbstractButton b) {
		super.remove(b);
        //clean up after ourselves
        b.setFocusTraversalKeysEnabled(true);
        b.removeKeyListener(m_keyHandler);
	}
	
	/**
	 * An iterator for the buttons in this group
     * <p>This elimates the need to use the Enumeration
     * in the original ButtonGroup</p>
	 * @return Iterator
	 */
	public Iterator iterator() {
		return buttons.iterator();
	}
	
	/**
	 * Gets the button that is next from the button supplied
	 * returns null if the supplied button is the last in the group,
	 * or no button was supplied, or it is not part of the group 
	 * @param b, AbstractButton for which the next button is to be determined
	 * @return AbstractButton, the button next to the one supplied
	 */
	AbstractButton next(AbstractButton b) {
		int idx=buttons.indexOf(b);
		if (idx!=-1 && idx+1<buttons.size()) {
			return (AbstractButton) buttons.get(++idx);
		} else {
			return null;
		}
		
	}
	
	/**
	 * Gets the button that is previous to the button supplied
	 * returns null if the supplied button is the first in the group,
	 * or no button was supplied, or it is not part of the group 
	 * @param b, AbstractButton for which the pervius button is to be determined
	 * @return AbstractButton, the button previous from the one supplied
	 */
	AbstractButton previous(AbstractButton b) {
		int idx=buttons.indexOf(b);
		if (idx>0) {
			return (AbstractButton) buttons.get(--idx);
		} else {
			return null;
		}
	}
	
	/**
	 * Returns the toolTipText.
	 * @return String
	 */
	public String getToolTipText() {
		return m_toolTipText;
	}

	/**
	 * Sets the toolTipText.
	 * @param toolTipText The toolTipText to set
	 */
	public void setToolTipText(String toolTipText) {
		m_toolTipText = toolTipText;
	}

	
    /**
     * Sets all the buttons in this group to enabled or disabled.
     * @param b, true for enabled, false for disabled
     * @since Framework 0.31
     */
	public void setEnabled(boolean b) {
		Iterator it=iterator();
		while (it.hasNext()) {
			((AbstractButton) it.next()).setEnabled(b);
		}
	}
	
	/** 
	 * set all buttons in this group to visible or invisible
	 * @param b, true for visible, for for invisible
	 * @since Framework 0.35
	 */
	public void setVisible(boolean b) {
		Iterator it=iterator();
		while (it.hasNext()) {
			((AbstractButton) it.next()).setVisible(b);
		}
	}
}
