/* 	GuiItemFactory.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Jul 6, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.swingext.gui;

import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
/** Factory class to create Swing components mostly from an Action
 *  <UL>Through UIMananager some options will influence the Components created.
 *  <LI><code>GuiItem.MenuItem.showIcon</code> will define if the menu item shows the Icon from the Action</LI>
 *  <LI><code>GuiItem.Button.showIcon</code> will define if a button shows the Icon from the Action</LI>
 *  <LI><code>GuiItem.Button.showText</code> will define if a button shows the Text from the Action (default to true)</LI>
 *  </UL>
 *  <UL>These Factory methods where created to correct some issues with
 *  either the components or setting an action to it.
 *  <LI>{@link JMenu} does not allow setting the mnemonic directly.</LI>
 *  <LI>{#link JMenuItem} does not set the accelerator from an Action</LI>
 *  </UL> 
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [Jul 6, 2007]
 */
public class GuiItemFactory {

    /**
     * Private, while it does not have to be instantiated
     */
    private GuiItemFactory() {
    }
    
    /**
     * Create a JMenu with a mnemonic
     * @param label String, the label for this menu
     * @param mnemonic char, the mnemonic for this JMenu
     * @return JMenu, the menu created
     * @see JMenuItem
     */
    public final static JMenu createMenu(String label, char mnemonic) {
        JMenu menu= new JMenu(label);
        menu.setMnemonic(mnemonic);
        return menu;
    }
    
    /**
     * Create a JMenuItem on the basis of an {@link Action} and add it to the parent.
     * <p>If {@link UIManager} property <code>GuiItem.MenuItem.showIcon</code> is 
     * set the JMenuItem will show an Icon if it is part of the Action.</p>
     * <p>The accelerator is copied from the Action seperatly, because it is not used
     * from the Action directly.</p>  
     * <p><i>Note: While JMenu is just another JMenuItem you can add JMenuItem's to 
     * any JMenu, or to a JMenuItem</i></p> 
     * @param parent JMenuItem, the menu item to add this menu item to. 
     * @param action Action, the action to create the JMenuItem from.
     * @return JMenuItem, the menu item created from the Action
     */
    public final static JMenuItem createMenuItem(JMenuItem parent, Action action) {
        JMenuItem item = new JMenuItem(action);
        item.setAccelerator( (KeyStroke) action.getValue(Action.ACCELERATOR_KEY));
        boolean showIcon  = UIManager.getBoolean("GuiItem.MenuItem.showIcon");
        if (!showIcon) {
            item.setIcon(null);
        }
        parent.add(item);
        return item;
    }
}
