/* 	ScrollablePanel.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-mrt-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Jun 21, 2007	dennis@brains2b.nl      calculate preferredSize including the length of a possible scrollbar		
*/
package org.brains2b.swingext.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

/** Panel which implements the Scrollable interface for multi continous forms
 *  <p>Adding panels which implement the SelectablePanel interface make it 
 *  easy to select a panel in this scrollable list of panels</p>
 *  <p>Scrolling can be set to be either HORIZONTAL or VERTICAL, the other dimension
 *  will thereby be fixed on the size of the added panels.
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [0.10] [23-mrt-2007]
 * @see SelectablePanel
 */
public class ScrollablePanel extends JPanel implements Scrollable {
    
    class FocusHandler implements FocusListener {
        public void focusGained(FocusEvent e) {
           if (getRoot(e.getOppositeComponent())==null) {
               boolean needRepaint=false;
               for (int i=0;i<getComponentCount();i++) {
                   if (getComponent(i) instanceof SelectablePanel) {
                       needRepaint=true;
                       ((SelectablePanel) getComponent(i)).setSelected(false);
                   }
               }
               if (needRepaint) {
                   ScrollablePanel.this.repaint();
               }
           }
           Component root =getRoot(e.getComponent());
           m_selectedIndex=findIndex(root);
           if (root instanceof SelectablePanel) {
               ((SelectablePanel) root).setSelected(true);
               ScrollablePanel.this.repaint();
           }
           
        }
        
        public void focusLost(FocusEvent e) {
           Component root =getRoot(e.getComponent());
           Component oppRoot =getRoot(e.getOppositeComponent());
           if (root!=null && oppRoot!=null && root!=oppRoot) {
              m_selectedIndex=-1;
              if (root instanceof SelectablePanel) {
                ((SelectablePanel) root).setSelected(false);
                ScrollablePanel.this.repaint();
              }
           }
        }
    }
    
    public final static int HORIZONTAL = SwingConstants.HORIZONTAL;
    public final static int VERTICAL = SwingConstants.VERTICAL;
    
    private int m_orientation;
    private int m_selectedIndex=-1;
    private Dimension m_scrollableSize;
    private FocusHandler m_focusHandler;
    
    public ScrollablePanel(int orientation) {
        super();
        m_orientation = orientation;
        init();
    }
    
    /**
     * {@inheritDoc}
     * <p>Adjust the Width of the Dimension to add
     * the width of a ScrollBar.</p>
     */
    public Dimension getPreferredSize() {
        Dimension d = super.getPreferredSize();
        if (getScrollableTracksViewportWidth()) {
            d.width+=UIManager.getInt("ScrollBar.width");
        }
        return d;
    }
    
    /**
     * {@inheritDoc}
     */
    public Dimension getPreferredScrollableViewportSize() {
        if (m_scrollableSize==null) {
            m_scrollableSize= getPreferredSize();
        }
        return m_scrollableSize;
    }

    /**
     * {@inheritDoc}
     */
    public int getScrollableBlockIncrement(Rectangle visibleRect,
            int orientation, int direction) {
         if (m_orientation==VERTICAL) {
            return visibleRect.height;
        } else {
            return visibleRect.width;
        }
    }

    /**
     * {@inheritDoc}
     */
    public boolean getScrollableTracksViewportHeight() {
        return (m_orientation==HORIZONTAL);
    }

    /**
     * {@inheritDoc}
     */
    public boolean getScrollableTracksViewportWidth() {
        return (m_orientation==VERTICAL);
    }

    /**
     * {@inheritDoc}
     */
    public int getScrollableUnitIncrement(Rectangle visibleRect,
            int orientation, int direction) {
        if (getComponentCount()==0) {
            return 0;
        }
        if (m_orientation==VERTICAL) {
            return getComponent(0).getSize().height;
        } else {
            return getComponent(0).getSize().width;
        }
    }
    
    /**
     * {@inheritDoc}
     */
    protected void addImpl(Component comp, Object constraints, int index) {
        
        if (comp instanceof Container) {
            for (int i=0;i< ((Container) comp).getComponentCount();i++) {
                Component cmp = ((Container) comp).getComponent(i);
                if (cmp.isFocusable()) {
                    cmp.addFocusListener(m_focusHandler);
                }
            }
        } else if (comp.isFocusable()) {
            comp.addFocusListener(m_focusHandler);
        }
        super.addImpl(comp, constraints, index);
    }

    /**
     * {@inheritDoc}
     */      
    public void remove(int index) {
        if (index==m_selectedIndex) m_selectedIndex=-1;
        super.remove(index);
    }
    
    /**
     * The Panel selected in this Scroll list
     * @return int, the component index of the Panel selected
     */
    public int getSelectedIndex() {
        return m_selectedIndex;
    }

    /**
     * Set a panel in this Scroll list as selected
     * @param selectedIndex, the component index of the Panel to select
     */
    public void setSelectedIndex(int selectedIndex) {
        if (m_selectedIndex!=selectedIndex) {
            if (getComponent(selectedIndex) instanceof SelectablePanel) {
                ((SelectablePanel) getComponent(selectedIndex)).setSelected(true);
            }
            if (m_selectedIndex!=-1 && getComponent(m_selectedIndex) instanceof SelectablePanel) {
                ((SelectablePanel) getComponent(m_selectedIndex)).setSelected(false);
            }
        }
        m_selectedIndex = selectedIndex;
        
    }

    private final int findIndex(Component cmp) {
        for (int i=0;i<getComponentCount();i++) {
            if (cmp==getComponent(i)) return i;
        }
        return -1;
    }
    
    private final Component getRoot(Component cmp) {
        if (cmp==null) return null;
        if (cmp.getParent() == this) {
            return cmp;
        }
        return getRoot(cmp.getParent());
        
    }
    
    private void init() {
        m_focusHandler = new FocusHandler();
        addFocusListener(m_focusHandler);
        setLayout(new BoxLayout(this,m_orientation==HORIZONTAL?BoxLayout.X_AXIS:BoxLayout.Y_AXIS));
    }
    
}
