/* 	DateValidator.java
 *	Copyright (c) 2002-2007 Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 09/19/2002	dennis@brains2b.nl	Changes for setting year correctly
 * 22-04-2003	dennis@brains2b.nl	Check if date entered is valid and no rounding is done by Calendar
 * 03-07-2003	dennis@brains2b.nl	Fixed problems with years that are absolutely smaller then 100
 * 23-09-2003	dennis@brains2b.nl	Fixed locale case problem.
 * 									Implemented localized message
 * 01-06-2006	dennis@brains2b.nl	Support for 8 digit mask yyyyMMdd, implemented getString(Class c)
 * 02-09-2006	dennis@brains2b.nl	Added support for US customers entering dates in MMddyy MMddyyyy
 * 18-06-2007   dennis@brains2b.nl  Disallow lenient for inputting data, while it iterfers with mask
 */
package org.brains2b.swingext.mask;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JTextField;

import org.brains2b.util.DateHelper;

/** 
 * DateValidator is an InputValidator that functions like a display mask
 * for dates. 
 * <p>For the format of the Date any valid format as defined in SimpleDateFormat
 * can be used. The date inputted will be displayed conforming to this format.
 * <p><i>Note</i>You can display  time information through the mask, but the verify
 * does not allow inserting of times, so the current time will be displayed.
 * <UL>A date can be inserted in different formats based on the following rules:
 * <LI>a six digit String is interpreted as <code>ddMMyy</code>
 * <LI>a String with a non character separator is interpreted as <code>day-month-year</code>
 * <LI><code>day</code> can be either 1 or 2 digits, with or without a leading 0
 * <LI><code>month</code> can be  either 1 or 2 digits, with or without a leading 0, a String representing
 * the long name or a short name of the month. The name of the month must be in the same
 * language for the Locale the DateValidator is instantiated with.
 * <LI><code>year</code> can be 1 or 2 digits, with or without a leading 0. This is interpreted as
 * a year without a century, so the current century is added. It can also be four digits and will
 * then be intrepeted as year including a century.
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.14.1 [18-06-2007]
 * @see SimpleDateFormat
 */
public class DateValidator extends InputVerifier implements InputValidator {
	

	//private Calendar m_cal=Calendar.getInstance();
	
	private String m_mask="dd-MMM-yyyy";
	private Locale m_locale=Locale.getDefault();
	private Exception m_ex;
	/**
	 * Constructor for DateValidator.
	 * <p>The default mask is set to 'dd-MMM-yyyy'
	 * <p>The default Locale is set to Locale.getDefault()
	 */
	public DateValidator() {
		super();
	}
	
	/**
	 * Constructor for DateValidator.
	 * <p>The default mask is set to 'dd-MMM-yyyy'
	 * @param locale Locale, the Locale that is used to
	 * determine months and display months
	 */
	public DateValidator(Locale locale) {
		super();
		m_locale=locale;
	}
	
	/**
	 * Constructor for DateValidator.
	 * @param locale Locale, the Locale that is used to
	 * determine months and display months
	 * @param mask String, the mask for displaying the String
	 */
	public DateValidator(Locale locale, String mask) {
		super();
		m_locale=locale;
		m_mask=mask;
	}
	

	/**
	 * @see javax.swing.InputVerifier#verify(JComponent)
	 */
	public boolean verify(JComponent input) {
		if (!(input instanceof JTextField)) {
			return false;
		}
		JTextField txt= (JTextField) input;
		String inTxt=txt.getText();
		if (inTxt==null || inTxt.length()==0) { //Regard it optional by default
			return true;
		}
		
		SimpleDateFormat sdf=new SimpleDateFormat(m_mask,m_locale);
		sdf.setLenient(false);
        try {
			
			sdf.parse(inTxt);
			return true;
		} catch (ParseException pex) {
			try {
				Date dt=findFormat(inTxt);
				txt.setText(sdf.format(dt));
				return true;
			} catch (MaskException mex) {
				m_ex=mex;
				return false;
			}
		}
			
	}

	private Date findFormat(String txt) throws MaskException {
		//Calendar cal=Calendar.getInstance();
		
        try {
            return DateHelper.toDate(txt,m_locale);
        } catch (NumberFormatException nfex) {
            throw new MaskException(MaskException.NOT_VALID_DATE);
        }
        
        /*int dd=0;
		int mm=-1;
		int yy=0;
		int ylength=2; //default otherwise set underneath
				
		if ( txt.length()==6 && StringHelper.baseOnly(txt).length()==6 ) {
			try {
				if (m_locale.equals(Locale.US)) {
					mm=Integer.parseInt(txt.substring(0,2))-1;
					dd=Integer.parseInt(txt.substring(2,4));
				} else {
					dd=Integer.parseInt(txt.substring(0,2));
					mm=Integer.parseInt(txt.substring(2,4))-1;
				}
				yy=Integer.parseInt(txt.substring(4,6));
			} catch (NumberFormatException nfex) {
				throw new MaskException(nfex.getMessage());
			}
		} else if ( txt.length()==8 && StringHelper.baseOnly(txt).length()==8 ) {
			try {
				if (m_locale.equals(Locale.US)) {
					mm=Integer.parseInt(txt.substring(0,2))-1;
					dd=Integer.parseInt(txt.substring(2,4));
					yy=Integer.parseInt(txt.substring(4,8));
				} else {
					yy=Integer.parseInt(txt.substring(0,4));
					mm=Integer.parseInt(txt.substring(4,6))-1;
					dd=Integer.parseInt(txt.substring(6,8));
				}
			} catch (NumberFormatException nfex) {
				throw new MaskException(nfex.getMessage());
			}
		} else {
			int pos=0;
			try {	 	
				for (int i=0;i<txt.length();i++) {
				 	if (!Character.isLetterOrDigit(txt.charAt(i))) {
						if (pos==0) {
				 			dd=Integer.parseInt(txt.substring(pos,i));
						 	pos=i+1;
						 } else {
						 	mm=getMonth(txt.substring(pos,i));
						 	pos=i+1;
					 	}
				 	}
				}
				
				yy=Integer.parseInt(txt.substring(pos));
				ylength=txt.length()-pos;
			} catch (NumberFormatException nfex) {
				throw new MaskException(MaskException.NOT_VALID_DATE);
			}
		}
		//FIX Months start at 0
		if (dd>0 & dd<32 & mm>=0 & mm<12 & yy>0) {
			if (yy<100 && ylength<3) {
					yy=yy+((m_cal.get(Calendar.YEAR)/100)*100);
				}
			m_cal.set(yy,mm,dd);
			// Strict implementation
			if (mm==m_cal.get(Calendar.MONTH) && dd==m_cal.get(Calendar.DAY_OF_MONTH)) {
				return m_cal.getTime();
			} else {
				throw new MaskException(MaskException.NOT_VALID_DATE);
			}
		} else {
			throw new MaskException(MaskException.NOT_VALID_DATE);
		}*/
	}		
	
	/*private int getMonth(String str) {
		int mm=0;
		try {
			mm=Integer.parseInt(str)-1;
			return mm;
		} catch (NumberFormatException ex) {
			DateFormatSymbols symb= new DateFormatSymbols(m_locale);
			if (str.length()>3) {
				String[] months=symb.getMonths();
				for (int i=0;i<months.length;i++) {
					if (str.equalsIgnoreCase(months[i])) {
						return i;
					} 
				}
			} else {
				String[] months=symb.getShortMonths();
				for (int i=0;i<months.length;i++) {
					if (str.equalsIgnoreCase(months[i])) {
						return i;
					} 
				}
			}		
		}	
		return -1;
	}*/
	/**
	 * @see InputValidator#getErrorMessage()
	 */
	public String getErrorMessage() {
		return m_ex.getMessage();
	}
	
    /**
     * @see org.brains2b.swingext.mask.InputValidator#getString(String)
     */
	public String getString(String txt) {
		return getString(txt,Date.class);
		
	}
	
	public String getString(String txt, Class c) {
		if (txt==null || txt.length()==0) return null;
		if (c==Date.class) {
			try {
				if (m_mask.equals("dd-MMM-yyyy")) return txt;
				SimpleDateFormat sdf=new SimpleDateFormat(m_mask);
				Date d= sdf.parse(txt);
				sdf.applyPattern("dd-MMM-yyyy");
				return sdf.format(d);
			} catch (ParseException pex) {
				pex.printStackTrace();
				return null;
			}
		} else {
			try {
				SimpleDateFormat sdf=new SimpleDateFormat(m_mask);
				Date d= sdf.parse(txt);
				sdf.applyPattern("yyyyMMdd");
				return sdf.format(d);
			} catch (ParseException pex) {
				pex.printStackTrace();
				return null;
			}
		} 
	}
	

    /**
     * @see org.brains2b.swingext.mask.InputValidator#getLocalizedMessage()
     */
    public String getLocalizedMessage() {
        return m_ex.getLocalizedMessage();
    }

}
