/* 	MaskedField.java
 * 
 * Created on: 02-10-2002
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002-2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 2-sep-03		dennis@brains2b.nl	If columns are not set, set them here.
 * 01-06-2006	dennis@brains2b.nl	Implemented the getString(Class) method
 * 05-04-2007   dennis@brains2b.nl  Implemented InputVerifierWrapper to make verifier possible next
 *                                  to validator 
 * 03-10-2007	dennis@brains2b.nl	Fix for setting an empty InputVerifier
 */
package org.brains2b.swingext.mask;

import java.util.ArrayList;
import java.util.Locale;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JTextField;

/** 
 * MaskedField extends JTextField as a convinience for implementing
 * different display masks through InputValidator
 * <p>Be sure to understand that an InputValidator is implemented
 * as setInputVerifier() for which there can be only one (grin!). So you cannot
 * implement another verifier on a MaskedField, other then writting a
 * Verifier that can contain multiple verifiers and implement them all there.
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.12 [03-10-2007]
 * @see org.brains2b.swingext.mask.InputValidator
 */
public class MaskedField extends JTextField {

    /**
     * Wrapper around InputVerifier making it possible
     * to have more then one InputVerifier.
     */
    class InputVerifierWrapper extends InputVerifier {

        private ArrayList m_verifiers= new ArrayList();
        
        public boolean verify(JComponent input) {
            for (int i=0;i<m_verifiers.size();i++) {
                boolean b= ((InputVerifier) m_verifiers.get(i)).verify(input);
                if (!b) return b;
            }
            return true;
        }
        
        public void addInputVerifier(InputVerifier verifier) {
           m_verifiers.add(verifier);       
        }
        
        public void removeInputVerifier(InputVerifier verifier) {
            m_verifiers.remove(verifier);
        }
        
    }
    
	private int m_type;
	private InputValidator m_validator;
	/** Constant for defining a TextValidator */
	public final static int TEXT   =0x01;
	/** Constant for defining a NumberValidator */
	public final static int NUMBER =0x02;
	/** Constant for defining a DateValidator */
	public final static int DATE   =0x03;

    /**
     * Constructor for MaskedField
     * @param mask String, the mask to use
     * @param type int, the type of InputValidator to use
     * can be TEXT, NUMBER or DATE
     */
	public MaskedField(String mask, int type) {
		super();
		m_type=type;
		if (getColumns()==0) {
			setColumns(mask.length());
		}
		switch (type) {
			case TEXT:
				setHorizontalAlignment(LEFT);
				m_validator=new TextValidator(mask);
				break;
			case NUMBER:
				setHorizontalAlignment(RIGHT);
				m_validator=new NumberValidator(mask);
				break;
			case DATE:
				m_validator=new DateValidator(Locale.getDefault(),mask);
				break;
		}	
		super.setInputVerifier((InputVerifier) m_validator);								
	}
	
	public MaskedField(InputValidator validator) {
		m_validator=validator;
		super.setInputVerifier((InputVerifier) validator);
		if (validator instanceof TextValidator) {
			m_type=TEXT;
		} else if (validator instanceof DateValidator) {
			m_type=DATE;
		}  else {
			m_type=NUMBER;
		}
	}

	/**
	 * Overwrite of setText to make sure the input complies to the Mask
	 * @see javax.swing.text.JTextComponent#setText(String)
	 */
	public void setText(String t) {
		if (t==null || !t.equals(super.getText())) {
			super.setText(t);
			if (m_validator!=null) {
				m_validator.verify(this);	
			}
		}
	}

    /**
     * Alternative to setText implemented to make sure
     * the String inserted can be processed by the mask.
     * 
     * @param s String, the String to set to the Field
     * @see org.brains2b.swingext.mask.NumberValidator#makeString
     */
	public void setString(String s) {
		if (m_validator instanceof NumberValidator) {
			s=((NumberValidator)m_validator).makeString(s);
		}
		setText(s);
	}
	
    /**
     * gets the String as formatted possibly without the mask
     * as defined in org.brains2b.mask.InputValidator#getString()
     * @return String, the String as suppleid by the InputValidator
     * @see org.brains2b.swingext.mask.InputValidator#getString
     */
	public String getString() {
		return m_validator.getString(getText());
	}
	
	
	/**
     * gets the String as formatted possibly without the mask
     * as defined in org.brains2b.mask.InputValidator#getString(String, Class)
     * @param c Class, define the type of representations which make sense for the types it will be stored in
     * @return String, the String as supplied by the InputValidator
     * @see org.brains2b.swingext.mask.InputValidator#getString(String, Class)
     */
	public String getString(Class c) {
		return m_validator.getString(getText(),c);
	}

    /**
     * set the Text to a MaskedField with the complete mask.
     * <p>Methods avoid verifying the mask, so use with caution.
     * @param txt
     */
	void setTextWithMask(String txt) {
		super.setText(txt);
	}
	
    /**
     * Returns the type of this MaskedField
     * <LI>DATE
     * <LI>NUMBER
     * <LI>TEXT
     * @return int the type of this MaskedField
     * @since Framework 0.40
     */
    public int getType() {
        return m_type;
    }
    
    /**
     * set InputVerifier on this MaskedField
     * by wrapping MaskedFields own InputValidator and the added one
     * to an InputVerifierWrapper
     */
    public void setInputVerifier(InputVerifier inputVerifier) {
    		if (inputVerifier == null) {
    			super.setInputVerifier(null);
    			return;
    		}
        if (getInputVerifier() instanceof InputVerifierWrapper) {
            ((InputVerifierWrapper) getInputVerifier()).addInputVerifier(inputVerifier);
            return;
        }
        if (getInputVerifier() instanceof InputValidator) {
            InputVerifierWrapper verifier = new InputVerifierWrapper();
            verifier.addInputVerifier(getInputVerifier());
            verifier.addInputVerifier(inputVerifier);
            inputVerifier = verifier;
        } 
        super.setInputVerifier(inputVerifier);
        
    }

}
