/* 	NumberValidator.java
 * 
 * 	Created on: 17-09-2002
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002-2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]			[Who]							[What]
 * 12-02-2003	dennis@brains2b.nl	rewrite of getMasked function to fix separator problem
 * 16-02-2003	dennis@brains2b.nl	fixed problem with get LeftPart not taking into account the length of the RighPart of the mask
 * 04-06-2003	dennis@brains2b.nl	Allow for negative numbers to end with negative sign
 * 									Display zero without negative sign
 * 23-09-2003	dennis@brains2b.nl	Return numbers from getString() without leading zero's
 * 									Implemented localized message support
 * 									Do not determine locale in a static manner
 * 08-11-2004	dennis@brains2b.nl	Fixed problem with hashes only
 * 01-06-2006	dennis@brains2b.nl	Implemented getString(String s, Class c)
 * 05-04-2007   dennis@brains2b.nl  Fixed problem with numbers without fraction in getNumberAsString
 * 22-06-2007   dennis@brains2b.nl  Fix for makeString on a real double
 * 08-10-2007	dennis@brains2b.nl	Fix voor fracties die te groot zijn.
 */
package org.brains2b.swingext.mask;

import java.util.Locale;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JTextField;

import org.brains2b.util.StringHelper;

/** 
 * NumberValidator is an InputValidator that functions like a display mask
 * for numbers. 
 * <p> for example: if you typed 50 using the default mask the displayed value would be 50,00
 * <BR> if you typed 1000,1 using the default mask the displayed value would be 1.000,10
 * <p>You can set the mask the input will comply to through the Constructor
 * <p> If the mask is not supplied the default "##.###9,99" is used.
 * <UL>Valid mask characters are:
 * <LI><b>#</b><tab><tab> Any number or blank if not supplied
 * <LI><b>9</b> <tab><tab>Any number or 0 if not supplied
 * <LI><b>comma,dot,space</b>Used as separators either for decimal or thousands.
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.24.2 [08-10-2007]
 */
public class NumberValidator extends InputVerifier implements InputValidator {

	private static char FRACTION=',';
	
	private static char THOUSANDSEP='.';
	
	private String m_mask="##.##9,99";
	private Exception m_ex;

	/**
	 * Constructor for NumberValidator.
	 * <p>Uses the default mask "##.##9,99"
	 */
	public NumberValidator() {
		super();
	}

	/**
	 * Constructor for NumberValidator.
	 * This constructor will use Locale.getDefault() as default 
	 * language.
	 * @param mask String, the mask to be used to display numbers 
	 * 	(see class description for mask characters)
	 */
	public NumberValidator(String mask) {
		this(mask,Locale.getDefault());
	}
	
	public NumberValidator(String mask, Locale locale) {
		setLocale(locale);
		setMask(mask);
	} 
	
	/**
	 * @see javax.swing.InputVerifier#verify(JComponent)
	 */
	public boolean verify(JComponent input) {
		boolean negative= false;
		if (!(input instanceof JTextField)) {
			return false;
		}
		JTextField txt= (JTextField) input;
		String inTxt=txt.getText();
		if (inTxt==null || inTxt.length()==0) { //Regard it optional by default
			return true;
		} else if (inTxt.charAt(0)=='-') {
			inTxt=inTxt.substring(1);
			negative=true;		
		} else if (inTxt.charAt(inTxt.length()-1)=='-') {
			inTxt=inTxt.substring(0,inTxt.length()-1);
			negative=true;
		}
				
		
		try {
			Integer.parseInt(StringHelper.baseOnly(inTxt));
		} catch (NumberFormatException nfex) {
			m_ex=new MaskException(MaskException.NOT_VALID_NUMBER);
			return false;
		}
			
		
		try {
			
			if (isMasked(inTxt) && !negative) {
				return true;
			} else {
				inTxt=(negative && !"0".equals(inTxt)?"-":"")+getMasked(inTxt);
				if (txt instanceof MaskedField) {
					((MaskedField)txt).setTextWithMask(inTxt);
				} else {
					txt.setText(inTxt);
				}
				
				return true;
			}
		} catch (MaskException mex) {
			m_ex = mex;
			return false;
		} 
		
	}

	private boolean isMasked(String txt) throws MaskException {
		int dif=m_mask.length()-txt.length();
		if (dif<0) return false;
		int mdx=m_mask.lastIndexOf(FRACTION);
		mdx=(mdx==-1?0:m_mask.length()-mdx);
		if (mdx>=txt.length()) return false;
		for (int i=0;i<txt.length();i++) {
			if (!isValid(txt.charAt(i),i+dif)) {
				return false;
			}
		}
		for (int i=0;i<dif;i++) {
			if (m_mask.charAt(i)=='9') return false;
		}
		return true;
	}
	
	
	private boolean isValid(char c, int i) {
		char m=m_mask.charAt(i);
			
		if( Character.isDigit(c) && m!='9' && m!='#') {
				return false;
		} else if (c=='.'&& m!='.') {
			return false;
		} else if (c==','&& m!=',') {
			return false;
		} else if (c==' ' && m!=' ' && m!='#' ) {
			return false;
		}
		return true;
	}

    private String getMasked(String txt) throws MaskException {
		StringBuffer sb=new StringBuffer();
		
		
		txt=txt.replace('.',FRACTION);
		int mdx=m_mask.lastIndexOf(FRACTION);
		if (mdx!=-1) {
			int idx=(txt.indexOf(FRACTION)!=-1?txt.indexOf(FRACTION):txt.length());
			sb.append(getRightPart(txt.substring(0,idx),m_mask.substring(0,mdx)));
			sb.append(getLeftPart(txt.substring(idx),m_mask.substring(mdx)));
    	} else {
    		sb.append(getRightPart(txt,m_mask));
		}
		return sb.toString();
    }
		
		
	private String getLeftPart(String part, String maskPart) throws MaskException{
		StringBuffer sb=new StringBuffer();
		final int maskBefore=m_mask.length()-maskPart.length();
		
		for (int mdx=0;mdx<maskPart.length();mdx++) {
			if (mdx<part.length() && isValid(part.charAt(mdx),maskBefore+mdx)) {
				sb.append(part.charAt(mdx));
			} else {
				char m=maskPart.charAt(mdx);
				if (m==FRACTION) {
					sb.append(FRACTION);
				} else if (m=='9') {
					sb.append('0');
				} else if (m!='#') {
					throw new MaskException(MaskException.NOT_VALID_NUMBER);
				}
			}
		}
		if (sb.length()==1 && sb.charAt(0)==FRACTION) { // If it only consists of the fraction
			return "";					
		} else {
			return sb.toString();
		}			
				
	}

	private String getRightPart(String part, String maskPart) throws MaskException{
		StringBuffer sb=new StringBuffer(part);
		int dif=maskPart.length()-part.length();
		
		if (dif<0) throw new MaskException(MaskException.VALUE_TO_LARGE);
		
		for (int mdx=maskPart.length()-1;mdx>=0;mdx--) {
			int idx=mdx-dif;
			if (idx<0 || !isValid(part.charAt(idx),mdx)) {
				char m=maskPart.charAt(mdx);
				if (idx>=0 &&(m==THOUSANDSEP || m==' ')) { //print this only if already started with the number
					sb.insert(idx+1,m);
				} else if (m!='9' && m!='#' && m!=THOUSANDSEP && m!=' ') {
					sb.insert(idx+1,m);
				} else if (m=='9') {
					sb.insert(idx+1,'0');
				} else if (idx>=0 && m!='#') {
					throw new MaskException(MaskException.NOT_VALID_NUMBER);
				}
				dif--;
			}
		}
		
		if (sb.toString().length()>maskPart.length()) {
			throw new MaskException(MaskException.VALUE_TO_LARGE);
		}
						
		return sb.toString();			
				
	}							
							
							
							
	/**
	 * @see InputValidator#getErrorMessage()
	 */
	public String getErrorMessage() {
		return m_ex.getMessage();
	}
	
	/**
     * @see InputValidator#getString(String)
     */
    public String getString(String txt) {
    		return getString(txt,Integer.class);
    }
    
    public String getString(String txt, Class c) {
    		if (txt==null) return null;
    		boolean negative=false;
    		if (txt!=null && txt.length()>0 && txt.charAt(0)=='-') negative=true;
    		
    		return (negative?"-":"") + getNumberAsString(txt,c);
    		
    	
    }

	
	/**
	 * makes a masked String for a number that is represented without a decimal place
	 * but does have one in the mask.
	 * <p>Assumed is that the value is multiplied with 10^x, where x is the position
	 * from the right of the decimal place.
	 * @param txt String, the text that should represent a number
	 * @return String the value as displayed with the Mask.
	 */
	public String makeString(String txt) {
		if (txt==null || txt.length()==0) return txt; //optional by default
    	boolean negative=false;
    	if (txt.charAt(0)=='-') {
    		txt=txt.substring(1);
    		negative=true;
    	}
    	if (txt.charAt(0)=='0' && m_mask.indexOf('9')==-1) {
    		return null;
    	}
    	txt = txt.replace('.', FRACTION);
        StringBuffer sb=new StringBuffer(txt);
    	
    	int dif=m_mask.indexOf(FRACTION);
    	if (dif!=-1 && txt.indexOf(FRACTION)==-1) {
    		dif=txt.length()-(m_mask.length()-dif)+1;
    		if (dif<=0) {
    			sb.insert(0,StringHelper.leftPad("0","0",Math.abs(dif)+1));
    			dif=1;
    		}
    		
    		if (dif>=0) {
	    		sb.insert(dif,FRACTION);
    		} 
        }  else if (dif==-1 && txt.indexOf(FRACTION)!=-1) {
            sb.delete(Math.max(0,txt.indexOf(FRACTION)-1), txt.length());
        }
    	if (negative) sb.insert(0,'-');
    	try {
	    	return getMasked(sb.toString());
    	} catch (MaskException mex) {
			m_ex=mex;
			return sb.toString();
		} 
    
    }
    
    /**
     * Transform a formatted String back to it's basic number
     * @since Framework 0.32
     */
    private String getNumberAsString(String txt,Class c) {
    	if (txt==null || txt.length()==0) return txt;
    	String result=null;
    	if (c==Integer.class || c==Long.class) {
    		result=StringHelper.baseOnly(txt);
    		result=String.valueOf(Integer.parseInt(result));
    	} else {
    		int idx=txt.indexOf(FRACTION);
            if (idx!=-1) {
    		  result=StringHelper.baseOnly(txt.substring(0,idx)) + txt.substring(idx);
    		  result=result.replace(FRACTION,'.');
            } else {
              result=StringHelper.baseOnly(txt);
            }
    	}
    	
    	return result;
    }
    /**
     * @see org.brains2b.swingext.mask.InputValidator#getLocalizedMessage()
     */
    public String getLocalizedMessage() {
        return m_ex.getLocalizedMessage();
    }

	private void setLocale(Locale loc) {
		if ("en".equals(loc.getLanguage())) {
			FRACTION='.';
			THOUSANDSEP=',';
		} else {
			FRACTION=',';
			THOUSANDSEP='.';
		}
	}
	
	private void setMask(String mask) {
		m_mask=mask;
	}
}
