/* 	TextValidator.java
 * 
 * Created on: 17-09-2002
 * Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 /* @#History
 * [When]		[Who]				[What]
 * 23-Sep-03	dennis@brains2b.nl		Implemented localized messages
 * 04-05-2004	dennis@brains2b.nl	Not all messages were localized. Added javadoc for
 * 									private methods
 * 16-04-2005	dennis@brains2b.nl	Allow for right align in TextFields with an > indicator
 * 01-06-2006	dennis@brains2.nl	Implemented getString(String s, Class c)
 * TODO How to treat fixed character prefixed by a ?
 *
 */
package org.brains2b.swingext.mask;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JTextField;

import org.brains2b.util.StringHelper;

/** 
 * TextValidator is an InputValidator that functions like a display mask
 * for Strings.
 * <p>The mask is a combination of mask characters and fixed indicators.
 * If you have a mask like X@-99-AAA?? and a value is inserted as XA01ABCD would be
 * displayed as XA-01-ABCD, if you inserted XAB9ABCD the verify would return false.
 * <UL>The following mask characters are valid:
 * <LI><b>?</b>Any letter, digit or blank, or not supplied
 * <LI><b>@</b>Any letter or digit
 * <LI><b>9</b>Any digit
 * <LI><b>A</b>Any letter
 * <LI>Other symbols will be seen as constant characters in the mask
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.14 [01-06-2006]
 */
public class TextValidator extends InputVerifier implements InputValidator {

	private String m_mask;
	private Exception m_ex;
	private boolean m_alignRight;
	
    /**
     * Constructor for TextValidator.
     * @param mask String, the Mask to use with this Validator
     * see the class description for allowed mask characters
     */
	public TextValidator(String mask) {
	    if (mask!=null && mask.length()!=0 && mask.charAt(0)=='>') {
	        m_alignRight=true;
	        mask=mask.substring(1);
	    }
		m_mask=mask;
	}
	/**
	 * @see javax.swing.InputVerifier#verify(JComponent)
	 */
	public boolean verify(JComponent input) {
		if (m_mask==null) {
			return true;
		}
		if (!(input instanceof JTextField)) {
			return false;
		}
		JTextField txt= (JTextField) input;
		String inTxt=txt.getText();
		if (inTxt==null || inTxt.length()==0) { //Regard it optional by default
			return true;
		}
		if (m_alignRight && inTxt.length()<m_mask.length()) {
		    inTxt=StringHelper.leftPad(inTxt," ",getMaskElementLength());
		}
		
		
		try {	
			if (isMasked(inTxt)) {
				return true;
			} else {
				inTxt=getMasked(inTxt);
				if (txt instanceof MaskedField) {
					((MaskedField)txt).setTextWithMask(inTxt);
				} else {
					txt.setText(inTxt);
				}
				return true;
			}
		} catch (MaskException mex) {
			m_ex=mex;
			return false;
		}

		
	}

	/** check if the txt <code>String</code> complies with the mask
	 * 
	 * @param txt String, the value to be tested against the mask
	 * @return boolean, true if the value complies, false otherwise
	 * @throws MaskException, thrown if the txt is longer than the mask
	 * value and cannot be compliant anyway. This skips a possible check
	 * to make the mask compliant done in <code>getMasked(String></code>
	 * @see #getMasked(String)
	 */
	private boolean isMasked(String txt) throws MaskException {
		//int dif=m_mask.length()-txt.length();
		if (txt.length()>m_mask.length()) { throw new MaskException (MaskException.DOESNT_COMPLY);}
		for (int i=0;i<txt.length();i++) {
			if (!isValid(txt.charAt(i),i)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * check if an individual character is valid at a given position
	 * @param c char, the character to check
	 * @param i int, the position of the character <code>c</code>
	 * @return boolean, true if the character at this position is valid, false 
	 * otherwise
	 */
	private boolean isValid(char c, int i) {
		char m=m_mask.charAt(i);
			
		if( Character.isDigit(c) && m!='@' && m!='?' && m!='9' ) {
				return false;
		} else if (Character.isLetter(c) && m!='@' && m!='?' && m!='A') {
			return false;
		} else if (c==' ' && m!='?' && m!=' ') {
			return false;
		} else if (c!=m && m!='@' && m!='?' && m!='9' && m!='A'  ) {
			return false;
		} 
		return true;
	}

	/**
	 * get the String which complies to the mask for this Validator
	 * @param txt String, the text which should be made compliant.
	 * @return String, the text which complies with the mask
	 * @throws MaskException, thrown if the supplied text cannot comply
	 * to the mask
	 */
	private String getMasked(String txt) throws MaskException {
		StringBuffer sb=new StringBuffer();
	
		if (txt.length()>=m_mask.length()) { throw new MaskException (MaskException.DOESNT_COMPLY);}
		int dif=0;
		
		for (int i=0;i<m_mask.length();i++) {
			char m=m_mask.charAt(i);
			
			if (i-dif<txt.length() && isValid(txt.charAt(i-dif),i)) {
				sb.append(txt.charAt(i-dif));
			} else if (m!='@' && m!='9' && m!='A' && m!='?') {
					sb.append(m);
					dif++;
			} else if (i-dif==txt.length() && m!='?') {
		 		throw new MaskException(MaskException.DOESNT_COMPLY);
			}
			
		}
		
		/*if (dif<m_mask.length()-txt.length()) {
			for (int i=txt.length()+dif;i<m_mask.length();i++) {
				if (!isValid(' ',i)){
					throw new MaskException(MaskException.DOESNT_COMPLY);
				}
			}
		}*/		
		return sb.toString();			
				
	}							
	/**
	 * @see org.brains2b.swingext.mask.InputValidator#getErrorMessage()
	 */
	public String getErrorMessage() {
		return m_ex.getMessage();
	}
	
	/**
	 * Implementation of getString(String) in this case only returns
	 * the baseCharacters [0-9][A-Z], no spaces, or symbols from the Mask.
     * @see org.brains2b.swingext.mask.InputValidator#getString(String)
     */
    public String getString(String txt) {
        return getString(txt,String.class);
    }
    
    public String getString(String txt, Class c) {
    		return StringHelper.baseOnly(txt);
    	}

    /**
     * @see org.brains2b.swingext.mask.InputValidator#getLocalizedMessage()
     */
    public String getLocalizedMessage() {
        return m_ex.getLocalizedMessage();
    }
    
    private int getMaskElementLength() {
        int cnt=0;
        for (int i=0;i<m_mask.length();i++) {
            char c=m_mask.charAt(i);
            if (c=='?' || c=='9' || c=='@' || c=='A') {
                cnt++;
            }
        }
        return cnt;
    }

}
